@@ -1,5 +1,8 @@
+# GNU Enterprise Common Library - Base Module
 #
-# This file is part of GNU Enterprise.
+# Copyright 2001-2005 Free Software Foundation
+#
+# This file is part of GNU Enterprise
 #
 # GNU Enterprise is free software; you can redistribute it
 # and/or modify it under the terms of the GNU General Public
@@ -16,19 +19,13 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2005 Free Software Foundation
-#
-# FILE:
-# __init__.py
-#
-# DESCRIPTION:
-"""
-GNUe base module.  All gnue.* modules depend on gnue.common, so import gnue.<whatever>" will cause gnue.common.apps to be loaded. This sets up a GNUe environment.
-"""
-#
-# NOTES:
-#
+# $Id: __init__.py 7281 2005-03-31 21:39:54Z reinhard $
 
+"""
+GNUe base module.  All gnue.* modules depend on gnue.common, so import
+gnue.<whatever>" will cause gnue.common.apps to be loaded. This sets up a GNUe
+environment.
+"""
 
-# Init stuff like _() and True/False (for python <2.2)
+# Init stuff like _()
 import gnue.common.apps as _init
