@@ -19,22 +19,25 @@
 #
 # Copyright 2001-2005 Free Software Foundation
 #
-# $Id: checktype.py 6851 2005-01-03 20:59:28Z jcater $
+# $Id: checktype.py 7553 2005-05-25 13:48:21Z reinhard $
+
 """
-Support for checking the type of variables
+Support for checking the type of variables.
 """
+
 from types import *
 
 import string
 import sys
 
-from gnue.common.apps import i18n
+from gnue.common.apps import errors, i18n
+
 
 # -----------------------------------------------------------------------------
-# Exceptions
+# Exception definition
 # -----------------------------------------------------------------------------
 
-class TypeError (Exception):
+class TypeError (errors.SystemError):
   """
   Raised when L{checktype} detects a wrong type.
 
@@ -77,7 +80,8 @@ class TypeError (Exception):
                  'expected': self.__stringify (self.expected),
                  'actual'  : self.__stringify (self.actual),
                  'value'   : repr (self.value)}
-    Exception.__init__ (self, message)
+    errors.SystemError.__init__ (self, message)
+
 
 # -----------------------------------------------------------------------------
 # Check type of a variable
@@ -89,19 +93,24 @@ def checktype (variable, validtype):
 
   This function is available as builtin function.
 
-  @param variable: the variable to check
-  @param validtype: the type, the class, or a list of types and classes that
-    are valid
+  @param variable: Variable to check.
+  @param validtype: Type, class, or a list of types and classes that are valid.
+  @raise TypeError: The variable has a type not listed in the valid types.
   """
   if isinstance (validtype, ListType):
     for t in validtype:
-      if isinstance (variable, t):
-        return
+      if t is None:
+        if variable is None:
+          return
+      else:
+        if isinstance (variable, t):
+          return
   else:
     if isinstance (variable, validtype):
       return
   raise TypeError, (variable, validtype)
 
+
 # -----------------------------------------------------------------------------
 # Module initialization
 # -----------------------------------------------------------------------------
@@ -109,6 +118,7 @@ def checktype (variable, validtype):
 import __builtin__  
 __builtin__.__dict__['checktype'] = checktype
 
+
 # -----------------------------------------------------------------------------
 # Self test code
 # -----------------------------------------------------------------------------
