@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2002-2004 Free Software Foundation
+# Copyright 2002-2005 Free Software Foundation
 #
 # FILE:
 # GroupBox.py
@@ -46,8 +46,8 @@ class GroupBox(Control):
     self.SetMethod("SYSPAINT", self.Paint)
 
   def Paint(self,v1,v2,v3):
-    Y = self._ABSY
-    X = self._ABSX
+    Y = self.Y
+    X = self.X
     H = self.H
     W = self.W
     _paintBorder(self, Y, X, H, W, caption = self.CAPTION, color=self.COLOR)
