@@ -1,10 +1,13 @@
+# GNU Enterprise Common Library - Generic DBSIG2 database driver
+#
+# Copyright 2000-2005 Free Software Foundation
 #
 # This file is part of GNU Enterprise.
 #
 # GNU Enterprise is free software; you can redistribute it
 # and/or modify it under the terms of the GNU General Public
 # License as published by the Free Software Foundation; either
-# version 2, or(at your option) any later version.
+# version 2, or (at your option) any later version.
 #
 # GNU Enterprise is distributed in the hope that it will be
 # useful, but WITHOUT ANY WARRANTY; without even the implied
@@ -16,13 +19,24 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2005 Free Software Foundation
-#
+# $Id: __init__.py 7623 2005-06-20 11:30:37Z reinhard $
+
+"""
+Generic Database driver plugin for SQL based backends using a DBSIG2 compatible
+Python module.
+"""
 
-# Indicate that this is no plugin
 __noplugin__ = True
 
-from Connection import *
-from DataObject import *
+
+# =============================================================================
+# Driver info
+# =============================================================================
+
+class DriverInfo:
+  name = "SQL based backends"
+
+
+from Behavior import *
 from ResultSet import *
-from RecordSet import *
+from Connection import *
