@@ -1,10 +1,13 @@
+# GNU Enterprise Common Library - Database drivers
+#
+# Copyright 2000-2005 Free Software Foundation
 #
 # This file is part of GNU Enterprise.
 #
 # GNU Enterprise is free software; you can redistribute it
 # and/or modify it under the terms of the GNU General Public
 # License as published by the Free Software Foundation; either
-# version 2, or(at your option) any later version.
+# version 2, or (at your option) any later version.
 #
 # GNU Enterprise is distributed in the hope that it will be
 # useful, but WITHOUT ANY WARRANTY; without even the implied
@@ -16,8 +19,8 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2005 Free Software Foundation
-#
+# $Id: __init__.py 7631 2005-06-20 15:43:48Z reinhard $
+
 """
-Database-specific datasource drivers
+Database driver plugins.
 """
