@@ -53,9 +53,6 @@ class GObj(GTriggerCore, ParserObj):
   def __init__(self, *args, **parms):
     GTriggerCore.__init__(self)
     ParserObj.__init__(self, *args, **parms)
-    # This should be ok to remove as glimpse doesn't show it being used anywhere
-    # and it's messing up the common namespace property logic
-    #self.__properties__ = {}
     self._inits = []
 
   def buildObject(self, **params):
@@ -74,14 +71,6 @@ class GObj(GTriggerCore, ParserObj):
     """
     self.phaseInit(self.buildObject(**params))
 
-  def getParent(self):
-    """
-    Returns the immediate parent of an object instance in a GObj tree.
-
-    @return: The parent of the object in the GObj tree.
-    @rtype: any
-    """
-    return self._parent
 
   #
   # phaseInit
@@ -92,7 +81,7 @@ class GObj(GTriggerCore, ParserObj):
 
     Typically called from within a GParser instance.  phaseInit interates
     thru the GObj tree as many times as necessary to fully initialize the
-    tree.  It determines the number of interations to perform during it's
+    tree.  It determines the number of iterations to perform during it's
     first pass down the tree.
 
     phaseInit looks for a private list variable called _inits that
@@ -112,6 +101,10 @@ class GObj(GTriggerCore, ParserObj):
     initialize itself.  This type of logic cannot be places into an __init__
     as the children may not be loaded yet or may not yet have the needed
     information.
+    
+    @type iterations: integer
+    @param iterations: Limits the number of passes to the specified number.
+                       This doesn't appear to be used anywhere.
     """
     if iterations == 0:
       iterations = self.maxInits()
@@ -119,9 +112,15 @@ class GObj(GTriggerCore, ParserObj):
       self._phaseInit(phase)
 
   def _phaseInit(self,phase):
+    """
+    Used internally by phaseInit to walk through the
+    object tree initializing objects.
+    """
 
 ## TODO: Below is a call-less recursive version of
 ## TODO: phaseInit.  Needs to be profiled both ways.
+## TODO: Profile tests have shown this to be .001 
+## TODO: cpu seconds faster per call
 ##    stack = [self]
 ##    while stack:
 ##      object = stack.pop()
@@ -131,7 +130,10 @@ class GObj(GTriggerCore, ParserObj):
 ##        init = object._inits[phase]
 ##      except IndexError:
 ##        continue
-##      init()
+##      try:
+##        init()
+##      except TypeError:
+##        continue
 
     inits = self._inits
     if (len(inits) > phase) and inits[phase]:
@@ -146,36 +148,52 @@ class GObj(GTriggerCore, ParserObj):
       initter(phase)
 
 
-  # This function is called after the parsers have completely
-  # constructed. All children should be in place and
-  # attributes and content should be set at this point.
-  # Return the number of phaseInit iterations your object will
-  # need.
-  #
-  # NOTE: Do not initialize datasources, etc at this point.
-  #       This is only so content can be set, etc, after
-  #       loading from XML.
-  #
   def _buildObject(self):
+    """
+    # This function is called after the parsers have completely
+    # constructed. All children should be in place and
+    # attributes and content should be set at this point.
+    # Return the number of phaseInit iterations your object will
+    # need.
+    #
+    # NOTE: Do not initialize datasources, etc at this point.
+    #       This is only so content can be set, etc, after
+    #       loading from XML.
+    #
+    
+    """
     return len(self._inits)
 
-  #
-  # maxInits functions
-  #
-  # maxInits returns the maximum size of all the _inits
-  # list from this object or it's children
-  #
   def maxInits(self):
+    """
+    maxInits returns the maximum size of all the _inits
+    list from this object or it's children
+    """
     self._initCount = 0
     self.walk(self._maxInitsWalker)
     return self._initCount
 
   def _maxInitsWalker(self, object):
+    """
+    The function passed to the tree walker to extract the
+    length of the _inits list.  Used by maxInits.
+    """
     if hasattr(object,'_inits'):
       self._initCount = max(self._initCount,len(object._inits))
 
 
   def getChildrenAsContent(self):
+    """
+    Returns the content of any GContent objects that are children 
+    of this object.  
+    
+    @rtype: string or binary(?)
+    @return: The contents of the children 
+    """
+
+    # This is down here to avoid recursive importing
+    from gnue.common.definitions.GBinary import GBinary
+
     content = ""
     for child in self._children:
       if isinstance(child, GContent):
@@ -203,9 +221,14 @@ class GObj(GTriggerCore, ParserObj):
   #
   # addChild
   #
-  # Adds an object to an instances list of children
   #
   def addChild(self, child):
+    """
+    Adds an object to an instances list of children
+    
+    @type child: GObj derived class
+    @param child: The object to add.
+    """
     self._children.append(child)
 
   def getXmlTag(self, stripPrefixes=None):
@@ -218,7 +241,6 @@ class GObj(GTriggerCore, ParserObj):
                           the gnue-forms objects.  If set to None (the default)
                           then the behaviour will be the old style which
                           removes the first two characters from the type.
-    @type multipler: list of strings
     @return: The xml tag to use
     @rtype: string
     """
@@ -366,6 +388,10 @@ class GObj(GTriggerCore, ParserObj):
 
 
   def walk(self, function, *args, **parms):
+    """
+    Function that recursivly walks down through a tree of GObj 
+    instances and applies a function to them.
+    """
     function(self, *args, **parms)
     for child in self._children:
       if isinstance(child, GObj):
@@ -379,7 +405,7 @@ class GObj(GTriggerCore, ParserObj):
     if includeSelf:
       parentObject = self
     else:
-      parentObject = self._parent
+      parentObject = self.getParent ()
 
     while 1:
       if parentObject == None:
@@ -388,19 +414,18 @@ class GObj(GTriggerCore, ParserObj):
         return parentObject
 
       # If passed a type of NONE it finds the top object in the tree
-      if not type and not parentObject._parent:
+      if not type and not parentObject.getParent ():
         return parentObject
 
-      parentObject = parentObject._parent
+      parentObject = parentObject.getParent ()
 
-  #
-  # findChildOfType
-  #
-  # Moves downward though the children of an object till
-  # it finds the child of the specified type
-  #
   def findChildOfType(self, type, includeSelf=1, allowAllChildren=0):
-
+    """
+    Moves downward though the children of an object till
+    it finds the child of the specified type
+  
+    """
+  
     if includeSelf and self._type == type:
       return self
 
@@ -417,13 +442,10 @@ class GObj(GTriggerCore, ParserObj):
     return None
 
 
-  #
-  # findChildrenOfType
-  #
-  # find all children of a specific type
-  #
   def findChildrenOfType(self, type, includeSelf=1, allowAllChildren=0):
-
+    """
+    find all children of a specific type
+    """
     rs = []
 
     if includeSelf and self._type == type:
@@ -443,13 +465,11 @@ class GObj(GTriggerCore, ParserObj):
     return rs
 
 
-  #
-  # getDescription
-  #
-  # Return a useful description of this object
-  # Used by designer clients
-  #
   def getDescription(self):
+    """
+    Return a useful description of the object.
+    Currently used by GNUe Designer.
+    """
     if hasattr(self,'_description'):
       return self._description
     elif hasattr(self,'name'):
@@ -457,7 +477,9 @@ class GObj(GTriggerCore, ParserObj):
     else:
       return self._type[2:] + " (%s)" % self._type[2:]
 
-  # Hooks
+  # ===========================================================================
+  # Hooks - I haven't a clue yet
+  # ===========================================================================
   def __getitem__(self, key):
     return self.__dict__[key.replace(':','__')]
 
@@ -498,9 +520,10 @@ class ParserMultiplexor(ParserObj):
       if attr not in ('_buildObject','getClass') and attr[:2] != '__':
         newObj.__dict__[attr] = value
 
-    if self._parent:
-      self._parent._children[self._parent._children.find(self)] = newObj
-    return newObj._buildObject(self)
+    parent = self.getParent ()
+    if parent:
+      parent._children [parent._children.find (self)] = newObj
+    return newObj._buildObject (self)
 
 
   # This should return a GObj-based class
@@ -508,5 +531,53 @@ class ParserMultiplexor(ParserObj):
     raise "Virtual method not implemented"
 
 
-# This is down here to avoid recursive importing
-from gnue.common.definitions.GBinary import GBinary
+# =============================================================================
+# Collection class where the children in a diff won't get divided
+# =============================================================================
+
+class GUndividedCollection (GObj):
+  """
+  """
+
+  # ----------------------------------------------------------------------------
+  # Constructor
+  # ----------------------------------------------------------------------------
+
+  def __init__ (self, *args, **kwargs):
+
+    GObj.__init__ (self, *args)
+    self.buildObject (**kwargs)
+
+
+  # ----------------------------------------------------------------------------
+  # Build a difference-tree to the given object tree
+  # ----------------------------------------------------------------------------
+
+  def diff (self, goal, maxIdLength):
+
+    result = ParserObj.diff (self, goal, maxIdLength)
+
+    if result is not None:
+      for item in result._children [:]:
+        # We cannot change a child of an immutable collection directly, instead
+        # we remove the original and add the changed one.
+        if item._action == 'change':
+          result._children.remove (item)
+
+          # Add a copy of the 'old' item to be 'removed'
+          for old in self._children:
+            if old._id_ (maxIdLength) == item._id_ (maxIdLength):
+              add = old.__class__ (result)
+              add.assign (old, True)
+              add.walk (self._diffActionWalker_, "remove")
+              break
+
+          # and one copy of the 'new' item to be 'added'
+          for new in goal._children:
+            if new._id_ (maxIdLength) == item._id_ (maxIdLength):
+              add = new.__class__ (result)
+              add.assign (new, True)
+              add.walk (self._diffActionWalker_, "add")
+              break
+
+    return result
