@@ -165,6 +165,9 @@ def loadXMLObject(stream, handler, rootType, xmlFileType,
     gDebug (7, "Initializing the object tree starting at %s" %(object))
     object.phaseInit(dh._phaseInitCount)
 
+  # Break the cyclic reference in the content handler, so garbage collection is
+  # able to collect it
+  dh.parser = None
 
 
   return object
@@ -287,6 +290,7 @@ class xmlHandler(xml.sax.ContentHandler):
     ns, name = qtag
     attrs = {}
     loadedxmlattrs = {}
+    baseAttrs = {}
 
     if not ns or ns == self.default_namespace:
       #
@@ -444,6 +448,19 @@ class xmlHandler(xml.sax.ContentHandler):
     self.xmlStack.insert(0, object)
     self.nameStack.insert(0, name)
 
+    # processing trigger/procedure code from external files
+    for qattr in saxattrs.keys():
+      attrns, attr = qattr
+
+      if baseAttrs.has_key ('file') and attr == 'file':
+        textEncoding=gConfig('textEncoding')
+        handle = openResource(attrs[attr])
+        text = handle.read().decode(textEncoding)
+        handle.close()
+        
+        if self.xmlStack[0] != None:
+          GContent(self.xmlStack[0], text)
+
 
   #
   # Called by the internal SAX parser whenever
@@ -531,7 +548,7 @@ class GImportItem(GObj):
      #
      rv = self.__findImportItem(self, form, id)
      if rv != None:
-       rv._parent = self
+       rv.setParent (self)
        rv._IMPORTED = 1
        self._children.append(rv)
        #
@@ -595,7 +612,7 @@ class GImport(GObj):
           for child in form._children:
             if isinstance(child,instanceType) and \
                (importAll or child.name in importNames):
-              child._parent = self
+              child.setParent (self)
               child._IMPORTED = 1
               self._children.append(child)
               child._buildObject()
