@@ -104,9 +104,9 @@ class GTrigger(GObj):
     self.__call__ = self.dummyFunction
 
     if self.type != "NAMED":
-      if self._parent:
-        self._parent.associateTrigger( self.type, self )
-        self._triggerns.update(self._parent._localTriggerNamespace)
+      if self.getParent ():
+        self.getParent ().associateTrigger (self.type, self)
+        self._triggerns.update(self.getParent ()._localTriggerNamespace)
     else:
       self._root._triggerDictionary[self.name] = self
       self._triggerns.update(self._root._localTriggerNamespace)
