@@ -103,9 +103,9 @@ class GTrigger(GObj):
     self.__call__ = self.dummyFunction
 
     if self.type != "NAMED":
-      if self._parent:
-        self._parent.associateTrigger( self.type, self )
-        self._triggerns.update(self._parent._localTriggerNamespace)
+      if self.getParent ():
+        self.getParent ().associateTrigger (self.type, self)
+        self._triggerns.update (self.getParent ()._localTriggerNamespace)
     else:
       self._root._triggerDictionary[self.name] = self
       self._triggerns.update(self._root._localTriggerNamespace)
@@ -165,7 +165,7 @@ class GTrigger(GObj):
       if p._triggerGlobal == 1:
         p=None
       else:
-        p=p._parent
+        p=p.getParent ()
 
     method = cx.buildFunction (path, self._text)
     self.__call__ = method
@@ -188,8 +188,8 @@ class GTrigger(GObj):
   # dumpXML
   #
   #
-  def dumpXML(self, lookupDict, treeDump=None, gap=None,
-              textEncoding='<locale>',xmlnamespaces={}, stripPrefixes = None):
+  def dumpXML(self, lookupDict, treeDump=None, gap=None, xmlnamespaces={},
+              textEncoding='<locale>', stripPrefixes=None):
     """
     Dumps an XML representation of a trigger.
     used in saving.
@@ -305,12 +305,13 @@ class GTriggerExtension:
     """
     key = string.upper(key)
     if key in self._validTriggers.keys():
+      gDebug (9, 'Trigger %s on %s' % (key, repr (self)))
       if self._trigger.has_key(key):
         for function in self._trigger[key]:
           # TODO: do we need to call "updateNamespace" here?
           #   function.updateNamespace()
           try:
-            function(self)
+            return function(self)
           except AbortRequest:
             if not ignoreAbort:
               raise
@@ -336,6 +337,8 @@ def getXMLelements(updates={}):
                'Typecast': GTypecast.name },
             'type': {
                'Typecast': GTypecast.uppername },
+            'file': {
+               'Typecast': GTypecast.name },
             'src': {
                'Label': _('Source Trigger'),
                'References': 'trigger.name',
