@@ -18,7 +18,7 @@
 #
 # Copyright 2001-2005 Free Software Foundation
 #
-# $Id: Base.py 7057 2005-02-23 16:53:58Z johannes $
+# $Id: Base.py 7204 2005-03-14 19:16:03Z johannes $
 
 import sys
 import traceback
@@ -113,6 +113,19 @@ class BaseExecutionContext:
     raise ImplementationError, (self.__class__, 'bindFunction ()')
 
 
+  # ---------------------------------------------------------------------------
+  # Bind an element into the contexts builtin-namespace
+  # ---------------------------------------------------------------------------
+
+  def bindBuiltin (self, name, anElement):
+    """
+    Abstract: A descendant overrides this function to bind a given element into
+    the builtin-namespace of the context.
+    """
+
+    raise ImplementationError, (self.__class__, 'bindBuiltin ()')
+
+
   # ---------------------------------------------------------------------------
   # Create a new function instance 
   # ---------------------------------------------------------------------------
