@@ -37,16 +37,16 @@ from gnue.common.logic import language
 from gnue.common.logic.adapters import Base
 from gnue.common.logic.NamespaceCore import NamespaceElement
 
-try:
-  from spidermonkey import Runtime
-except:
-  print 'Spidermonkey python module not installed!'
-  print 'You can get it from http://wwwsearch.sourceforge.net/python-spidermonkey/!'
-  sys.exit(0)
-
 class LanguageAdapter(Base.BaseLanguageAdapter):
   def __init__(self):
-    self._rt = Runtime()
+    # Import down here so module remains importable (for eypdoc)
+    try:
+      import spidermonkey
+    except ImportError:
+      raise errors.AdminError (
+          "Spidermonkey python module not installed!\n"
+          "http://wwwsearch.sourceforge.net/python-spidermonkey/")
+    self._rt = spidermonkey.Runtime()
     
   def createNewContext(self):
     return ExecutionContext(self._rt)
