@@ -18,7 +18,7 @@
 #
 # Copyright 2001-2005 Free Software Foundation
 #
-# $Id: python.py 7057 2005-02-23 16:53:58Z johannes $
+# $Id: python.py 7204 2005-03-14 19:16:03Z johannes $
 
 
 import string
@@ -67,6 +67,7 @@ class PythonExecutionContext (Base.BaseExecutionContext):
 
     self._globalNS = {}
     self._localNS  = {}
+    self._builtins = {}
 
 
   # ---------------------------------------------------------------------------
@@ -111,6 +112,20 @@ class PythonExecutionContext (Base.BaseExecutionContext):
       self._localNS [name] = aFunction
 
 
+  # ---------------------------------------------------------------------------
+  # Bind an element into the contexts builtin-namespace
+  # ---------------------------------------------------------------------------
+
+  def bindBuiltin (self, name, anElement):
+    """
+    Bind the given element into the builtin-namespace.
+    @param name: name of the element within the builtin-namespace
+    @param anElement: element to be bound into the builtin-namespace
+    """
+
+    self._builtins [name] = anElement
+
+
   # ---------------------------------------------------------------------------
   # Create a function
   # ---------------------------------------------------------------------------
@@ -129,11 +144,9 @@ class PythonExecutionContext (Base.BaseExecutionContext):
     context.
     """
 
-    for key in self._globalNS.keys ():
-      del self._globalNS [key]
-
-    for key in self._localNS.keys ():
-      del self._localNS [key]
+    self._globalNS.clear ()
+    self._localNS.clear ()
+    self._builtins.clear ()
 
 
 # =============================================================================
@@ -159,7 +172,7 @@ class PythonFunction (Base.VirtualFunction):
     if not self.funcName:
       self.funcName = 'unnamed'
 
-    paramlist = string.join (['__namespace'] + \
+    paramlist = string.join (['__namespace', '__builtins'] + \
            [self.__identifier (key) for key in self._parameters.keys ()], ", ")
     text      = self._code.rstrip ()
 
@@ -179,12 +192,15 @@ class PythonFunction (Base.VirtualFunction):
 
       # add the functions header
       revisedCode  = "# -*- coding: utf-8 -*-\n"
+      revisedCode += "import __builtin__\n"
       revisedCode += "def %s (%s):\n" % (self.funcName, paramlist)
 
       # add the namespace transformation loop
       revisedCode += "  for __add in __namespace.keys ():\n"
       revisedCode += "    exec '%s = __namespace [\"%s\"]' % (__add, __add) in globals(), locals()\n"
-      revisedCode += "  del __add, __namespace\n\n"
+      revisedCode += "  for (name, item) in __builtins.items ():\n"
+      revisedCode += "    __builtin__.__dict__ [name] = item\n"
+      revisedCode += "  del __add, __namespace, __builtins\n\n"
 
       # add the original code
       for line in text.splitlines ():
@@ -218,7 +234,10 @@ class PythonFunction (Base.VirtualFunction):
       localNS = copy.copy (self._context._localNS)
       localNS.update (params)
       localNS ['__namespace'] = localNS
-      localNS ['abort'] = self.requestAbort
+      localNS ['__builtins'] = copy.copy (self._context._builtins)
+      # We do not use the context's bindBuiltin () method here to avoid another
+      # non-collectable reference
+      localNS ['__builtins'] ['abort'] = self.requestAbort
 
       try:
 
@@ -227,16 +246,12 @@ class PythonFunction (Base.VirtualFunction):
 
         exec self._compiled in self._context._globalNS, localNS
 
-        if localNS.has_key ('__result'):
-          return localNS ['__result']
-        else:
-          return None
+        return localNS.get ('__result')
 
       finally:
         # It's very importaint to release all references from the cloned
         # namespace here, otherwise garbage collection won't work very well !
-        for k in localNS.keys ():
-          del localNS [k]
+        localNS.clear ()
 
     except language.AbortRequest:
       # Pass through AbortRequests unchanged
