@@ -18,7 +18,7 @@
 #
 # Copyright 2001-2005 Free Software Foundation
 #
-# $Id: language.py 6892 2005-01-11 20:19:32Z jan $
+# $Id: language.py 7460 2005-04-22 14:53:23Z reinhard $
 
 import string
 
@@ -32,12 +32,6 @@ adapters = {}
 # Exceptions
 # =============================================================================
 
-# ---------------------------------------------------------------------------
-# The basis of all language errors
-# ---------------------------------------------------------------------------
-class Error (gException):
-  pass
-
 # -----------------------------------------------------------------------------
 # Python module not available
 # -----------------------------------------------------------------------------
