@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: client.py 7046 2005-02-20 14:43:17Z jan $
+# $Id: client.py 7460 2005-04-22 14:53:23Z reinhard $
 
 import string
 
@@ -41,13 +41,6 @@ def attach (interface, params):
 # Exceptions
 # =============================================================================
 
-# -----------------------------------------------------------------------------
-# The basis of all rpc.client exceptions
-# -----------------------------------------------------------------------------
-
-class Error (gException):
-  pass
-
 # -----------------------------------------------------------------------------
 # Requested adapter does not exist
 # -----------------------------------------------------------------------------
