@@ -41,7 +41,7 @@ CLIENT = 1      # ClientAdapter
 SERVER = 1      # ServerAdapter
 
 
-from gnue.common.rpc.drivers import GCommBase
+from gnue.common.rpc.drivers import Base
 from gnue.common.rpc import GComm
 
 import string, sys
@@ -50,7 +50,7 @@ import string, sys
 #
 # ClientAdapter
 #
-class ClientAdapter(GCommBase.Client):
+class ClientAdapter(Base.Client):
 
   def __init__(self, params):
     #print "Proxy Client Adapter created"
@@ -73,11 +73,11 @@ class ClientAdapter(GCommBase.Client):
 #
 # ServerAdapter
 #
-class ServerAdapter(GCommBase.Server):
+class ServerAdapter(Base.Server):
 
   def __init__(self, rpcdef, bindings, params):
     #print "Proxy Server Adapter created"
-    GCommBase.Server.__init__(self, rpcdef, bindings, params)
+    Base.Server.__init__(self, rpcdef, bindings, params)
 
   def raiseException(self, exception, message, event=None):
     raise exception, message
@@ -89,9 +89,9 @@ class ServerAdapter(GCommBase.Server):
 #
 # ProxyObject
 #
-class _ProxyObject(GCommBase.ProxyObject):
+class _ProxyObject(Base.ProxyObject):
   def __init__(self, adapter, attr, parent=None, **params):
-    GCommBase.ProxyObject.__init__(self, adapter, attr, parent=None, **params)
+    Base.ProxyObject.__init__(self, adapter, attr, parent=None, **params)
 
 
   # Server raised an exception...
