@@ -40,16 +40,17 @@ from gnue.common.rpc.drivers._helpers import ObjectLibrarian, ObjectEnabler
 
 import string, sys, os, posixpath, urllib, socket
 
-try:
-  import Pyro.naming
-  import Pyro.core
-  from Pyro.protocol import getHostname
-  from Pyro.errors import PyroError,NamingError
+def __initplugin__ ():
+  try:
+    import Pyro.naming
+    import Pyro.core
+    from Pyro.protocol import getHostname
+    from Pyro.errors import PyroError,NamingError
 
-except ImportError:
-  tmsg = _("\nUnable to load Pyro.  To use the Pyro interface, \n"
-           "please install pyro from:\n    http://pyro.sf.net/")
-  raise GComm.AdapterInitializationError, tmsg
+  except ImportError:
+    tmsg = _("\nUnable to load Pyro.  To use the Pyro interface, \n"
+             "please install pyro from:\n    http://pyro.sf.net/")
+    raise GComm.AdapterInitializationError, tmsg
 
 
 # Mapping from GRPC's datatype to XML-RPC datatypes
@@ -466,6 +467,3 @@ class ServerAdapter(Base.Server):
       result=1
       
     return result
-
-class RpcServiceProxy (Pyro.core.ObjBase):
-  pass
