@@ -37,23 +37,24 @@ CLIENT = 1      # ClientDriver
 SERVER = 1      # ServerDriver
 
 
-from gnue.common.rpc.drivers import GCommBase
+from gnue.common.rpc.drivers import Base
 
-try:
-  import soaplib
-except ImportError:
-  print _("\nUnable to load soaplib. To use the SOAP interface, "
-          "please install soaplib from:\n"
-          "    http://www.pythonware.com/products/soap/\n")
-  sys.exit()
+def __initplugin__ ():
+  try:
+    import soaplib
+  except ImportError:
+    print _("\nUnable to load soaplib. To use the SOAP interface, "
+            "please install soaplib from:\n"
+            "    http://www.pythonware.com/products/soap/\n")
+    sys.exit()
 
 
 
-class ClientDriver(GCommBase.BaseClientDriver): 
+class ClientDriver(Base.Client): 
   pass
 
 
-class ServerDriver(GCommBase.BaseServerDriver): 
+class ServerDriver(Base.Server): 
 
   def raiseException(self, exception, message):
     raise exception, message
