@@ -61,7 +61,7 @@ SERVER = 1      # ServerAdapter
 from gnue.common.rpc.drivers._helpers.AsyncSocketServer import AsyncSocketServer
 from gnue.common.rpc.drivers._helpers import ObjectLibrarian
 
-from gnue.common.rpc.drivers import GCommBase
+from gnue.common.rpc.drivers import Base
 from gnue.common.rpc import GComm
 
 import string
@@ -71,7 +71,7 @@ import socket
 #
 # ClientAdapter
 #
-class ClientAdapter(GCommBase.Client):
+class ClientAdapter(Base.Client):
 
   def __init__(self, params):
     try:
@@ -157,7 +157,7 @@ class ServerAdapter(AsyncSocketServer):
 #
 # ProxyObject
 #
-class _ProxyObject(GCommBase.ProxyObject):
+class _ProxyObject(Base.ProxyObject):
 
   def __getattr__(self, attr):
     self.__dict__[attr] = _ProxyObject(self._server, self, name=attr)
