@@ -19,13 +19,16 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: typeconv.py 7028 2005-02-15 13:50:52Z reinhard $
+# $Id: typeconv.py 7754 2005-07-27 14:38:16Z johannes $
 
 import time
 import types
+import datetime
 import mx.DateTime
 import xmlrpclib
 
+from gnue.common.utils import GDateTime
+
 # -----------------------------------------------------------------------------
 # Convert native Python type to xmlrpc's type
 # -----------------------------------------------------------------------------
@@ -63,6 +66,12 @@ def python_to_rpc (value, exception):
   elif isinstance (value, mx.DateTime.DateTimeDeltaType):
     return xmlrpclib.DateTime ((1, 1, 1, value.hour, value.minute,
                                int (value.second), 1, 1, 1))
+
+  elif isinstance (value, datetime.datetime) or \
+       isinstance (value, datetime.date) or \
+       isinstance (value, datetime.time):
+    return xmlrpclib.DateTime (value.isoformat ())
+
   # List
   elif isinstance (value, types.ListType):
     return [python_to_rpc (element, exception) for element in value]
@@ -78,6 +87,10 @@ def python_to_rpc (value, exception):
   elif isinstance (value, types.DictionaryType):
     result = {}
     for (key, val) in value.items ():
+      # Workaround for a bug xmlrpclib <= Python 2.3.4: No Unicode strings
+      # possible as dictionary keys.
+      if isinstance (key, types.UnicodeType):
+        key = key.encode ('utf-8')
       result [python_to_rpc (key, exception)] = python_to_rpc (val, exception)
     return result
 
@@ -119,10 +132,9 @@ def rpc_to_python (value, exception):
   elif isinstance (value, types.FloatType):
     return value
 
-  # Date/Time
+  # Date/Time/DateTime
   elif isinstance (value, xmlrpclib.DateTime):
-    # mx.DateTime.strptime doesn't work on windows
-    return mx.DateTime.mktime (time.strptime (value.value, '%Y%m%dT%H:%M:%S'))
+    return GDateTime.parseISO (value.value)
 
   # List
   elif isinstance (value, types.ListType):
