@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: server.py 7046 2005-02-20 14:43:17Z jan $
+# $Id: server.py 7460 2005-04-22 14:53:23Z reinhard $
 
 from types import *
 
@@ -103,13 +103,6 @@ from gnue.common.rpc.parser.Parser import loadDefinition
 # Exceptions
 # =============================================================================
 
-# -----------------------------------------------------------------------------
-# The basis of all rpc.client exceptions
-# -----------------------------------------------------------------------------
-
-class Error (gException):
-  pass
-
 # -----------------------------------------------------------------------------
 # The requested adapter does not exist...
 # -----------------------------------------------------------------------------
