@@ -112,7 +112,7 @@ def textToMeasurement(text, multiplier=1):
   @type text: string
   @param multiplier: A multiplier used to adjust the output
                      to values other inches.
-  @type multipler: number
+  @type multiplier: number
 
   @return: The converted value
   @rtype: number
